@@ -1,4 +1,4 @@
-/*	$OpenBSD: http.c,v 1.62 2022/05/24 09:22:45 claudio Exp $ */
+/*	$OpenBSD: http.c,v 1.68 2022/09/09 08:11:06 claudio Exp $ */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -70,7 +70,6 @@
 #define HTTP_USER_AGENT		"OpenBSD rpki-client"
 #define HTTP_BUF_SIZE		(32 * 1024)
 #define HTTP_IDLE_TIMEOUT	10
-#define HTTP_IO_TIMEOUT		(3 * 60)
 #define MAX_CONTENTLEN		(2 * 1024 * 1024 * 1024LL)
 #define NPFDS			(MAX_HTTP_REQUESTS + 1)
 
@@ -92,6 +91,7 @@ enum http_state {
 	STATE_RESPONSE_HEADER,
 	STATE_RESPONSE_DATA,
 	STATE_RESPONSE_CHUNKED_HEADER,
+	STATE_RESPONSE_CHUNKED_CRLF,
 	STATE_RESPONSE_CHUNKED_TRAILER,
 	STATE_WRITE_DATA,
 	STATE_IDLE,
@@ -356,7 +356,7 @@ recode_credentials(const char *userinfo)
 static void
 proxy_parse_uri(char *uri)
 {
-	char *host, *port = NULL, *cred, *cookie = NULL;
+	char *fullhost, *host, *port = NULL, *cred, *cookie = NULL;
 
 	if (uri == NULL)
 		return;
@@ -365,10 +365,10 @@ proxy_parse_uri(char *uri)
 		errx(1, "%s: http_proxy not using http schema", http_info(uri));
 
 	host = uri + 7;
-	if ((host = strndup(host, strcspn(host, "/"))) == NULL)
+	if ((fullhost = strndup(host, strcspn(host, "/"))) == NULL)
 		err(1, NULL);
 
-	cred = host;
+	cred = fullhost;
 	host = strchr(cred, '@');
 	if (host != NULL)
 		*host++ = '\0';
@@ -408,9 +408,12 @@ proxy_parse_uri(char *uri)
 		if ((cookie = strdup("")) == NULL)
 			err(1, NULL);
 
-	proxy.proxyhost = host;
+	if ((proxy.proxyhost = strdup(host)) == NULL)
+		err(1, NULL);
 	proxy.proxyport = port;
 	proxy.proxyauth = cookie;
+
+	free(fullhost);
 }
 
 /*
@@ -758,6 +761,19 @@ http_failed(struct http_connection *conn)
 	return DONE;
 }
 
+/*
+ * Called in case of connect timeout, try an alternate connection.
+ */
+static enum res
+http_connect_failed(struct http_connection *conn)
+{
+	assert(conn->state == STATE_CONNECT);
+	close(conn->fd);
+	conn->fd = -1;
+
+	return http_connect(conn);
+}
+
 /*
  * Call the function f and update the connection events based
  * on the return value.
@@ -1151,11 +1167,11 @@ http_redirect(struct http_connection *conn)
 static int
 http_parse_header(struct http_connection *conn, char *buf)
 {
-#define CONTENTLEN "Content-Length: "
-#define LOCATION "Location: "
-#define CONNECTION "Connection: "
-#define TRANSFER_ENCODING "Transfer-Encoding: "
-#define LAST_MODIFIED "Last-Modified: "
+#define CONTENTLEN "Content-Length:"
+#define LOCATION "Location:"
+#define CONNECTION "Connection:"
+#define TRANSFER_ENCODING "Transfer-Encoding:"
+#define LAST_MODIFIED "Last-Modified:"
 	const char *errstr;
 	char *cp, *redirurl;
 	char *locbase, *loctail;
@@ -1165,10 +1181,9 @@ http_parse_header(struct http_connection *conn, char *buf)
 	if (*cp == '\0')
 		return 0;
 	else if (strncasecmp(cp, CONTENTLEN, sizeof(CONTENTLEN) - 1) == 0) {
-		size_t s;
 		cp += sizeof(CONTENTLEN) - 1;
-		if ((s = strcspn(cp, " \t")) != 0)
-			*(cp + s) = 0;
+		cp += strspn(cp, " \t");
+		cp[strcspn(cp, " \t")] = '\0';
 		conn->iosz = strtonum(cp, 0, MAX_CONTENTLEN, &errstr);
 		if (errstr != NULL) {
 			warnx("Content-Length of %s is %s",
@@ -1178,6 +1193,7 @@ http_parse_header(struct http_connection *conn, char *buf)
 	} else if (http_isredirect(conn) &&
 	    strncasecmp(cp, LOCATION, sizeof(LOCATION) - 1) == 0) {
 		cp += sizeof(LOCATION) - 1;
+		cp += strspn(cp, " \t");
 		/*
 		 * If there is a colon before the first slash, this URI
 		 * is not relative. RFC 3986 4.2
@@ -1219,11 +1235,13 @@ http_parse_header(struct http_connection *conn, char *buf)
 	} else if (strncasecmp(cp, TRANSFER_ENCODING,
 	    sizeof(TRANSFER_ENCODING) - 1) == 0) {
 		cp += sizeof(TRANSFER_ENCODING) - 1;
+		cp += strspn(cp, " \t");
 		cp[strcspn(cp, " \t")] = '\0';
 		if (strcasecmp(cp, "chunked") == 0)
 			conn->chunked = 1;
 	} else if (strncasecmp(cp, CONNECTION, sizeof(CONNECTION) - 1) == 0) {
 		cp += sizeof(CONNECTION) - 1;
+		cp += strspn(cp, " \t");
 		cp[strcspn(cp, " \t")] = '\0';
 		if (strcasecmp(cp, "close") == 0)
 			conn->keep_alive = 0;
@@ -1232,6 +1250,7 @@ http_parse_header(struct http_connection *conn, char *buf)
 	} else if (strncasecmp(cp, LAST_MODIFIED,
 	    sizeof(LAST_MODIFIED) - 1) == 0) {
 		cp += sizeof(LAST_MODIFIED) - 1;
+		cp += strspn(cp, " \t");
 		free(conn->last_modified);
 		if ((conn->last_modified = strdup(cp)) == NULL)
 			err(1, NULL);
@@ -1257,8 +1276,10 @@ http_get_line(struct http_connection *conn)
 		return NULL;
 
 	len = end - conn->buf;
-	while (len > 0 && conn->buf[len - 1] == '\r')
+	while (len > 0 && (conn->buf[len - 1] == '\r' ||
+	    conn->buf[len - 1] == ' ' || conn->buf[len - 1] == '\t'))
 		--len;
+
 	if ((line = strndup(conn->buf, len)) == NULL)
 		err(1, NULL);
 
@@ -1273,7 +1294,6 @@ http_get_line(struct http_connection *conn)
 /*
  * Parse the header between data chunks during chunked transfers.
  * Returns 0 if a new chunk size could be correctly read.
- * Returns 1 for the empty trailer lines.
  * If the chuck size could not be converted properly -1 is returned.
  */
 static int
@@ -1283,12 +1303,8 @@ http_parse_chunked(struct http_connection *conn, char *buf)
 	char *end;
 	unsigned long chunksize;
 
-	/* empty lines are used as trailer */
-	if (*header == '\0')
-		return 1;
-
-	/* strip CRLF and any optional chunk extension */
-	header[strcspn(header, ";\r\n")] = '\0';
+	/* strip any optional chunk extension */
+	header[strcspn(header, "; \t")] = '\0';
 	errno = 0;
 	chunksize = strtoul(header, &end, 16);
 	if (header[0] == '\0' || *end != '\0' || (errno == ERANGE &&
@@ -1428,7 +1444,7 @@ again:
 				conn->bufpos = 0;
 			}
 			if (conn->chunked)
-				conn->state = STATE_RESPONSE_CHUNKED_TRAILER;
+				conn->state = STATE_RESPONSE_CHUNKED_CRLF;
 			else
 				conn->state = STATE_RESPONSE_DATA;
 			goto read_more;
@@ -1453,31 +1469,36 @@ again:
 		 * check if transfer is done, in which case the last trailer
 		 * still needs to be processed.
 		 */
-		if (conn->iosz == 0) {
-			conn->chunked = 0;
+		if (conn->iosz == 0)
 			conn->state = STATE_RESPONSE_CHUNKED_TRAILER;
-			goto again;
-		}
-
-		conn->state = STATE_RESPONSE_DATA;
+		else
+			conn->state = STATE_RESPONSE_DATA;
 		goto again;
-	case STATE_RESPONSE_CHUNKED_TRAILER:
+	case STATE_RESPONSE_CHUNKED_CRLF:
 		buf = http_get_line(conn);
 		if (buf == NULL)
 			goto read_more;
-		if (http_parse_chunked(conn, buf) != 1) {
+		/* expect empty line to finish a chunk of data */
+		if (*buf != '\0') {
 			warnx("%s: bad chunk encoding", http_info(conn->host));
 			free(buf);
 			return http_failed(conn);
 		}
 		free(buf);
-
-		/* if chunked got cleared then the transfer is over */
-		if (conn->chunked == 0)
-			return http_done(conn, HTTP_OK);
-
 		conn->state = STATE_RESPONSE_CHUNKED_HEADER;
 		goto again;
+	case STATE_RESPONSE_CHUNKED_TRAILER:
+		buf = http_get_line(conn);
+		if (buf == NULL)
+			goto read_more;
+		/* the trailer may include extra headers, just ignore them */
+		if (*buf != '\0') {
+			free(buf);
+			goto again;
+		}
+		free(buf);
+		conn->chunked = 0;
+		return http_done(conn, HTTP_OK);
 	default:
 		errx(1, "unexpected http state");
 	}
@@ -1677,7 +1698,7 @@ data_write(struct http_connection *conn)
 	/* all data written, switch back to read */
 	if (conn->bufpos == 0 || conn->iosz == 0) {
 		if (conn->chunked && conn->iosz == 0)
-			conn->state = STATE_RESPONSE_CHUNKED_TRAILER;
+			conn->state = STATE_RESPONSE_CHUNKED_CRLF;
 		else
 			conn->state = STATE_RESPONSE_DATA;
 		return http_read(conn);
@@ -1716,6 +1737,7 @@ http_handle(struct http_connection *conn)
 	case STATE_RESPONSE_HEADER:
 	case STATE_RESPONSE_DATA:
 	case STATE_RESPONSE_CHUNKED_HEADER:
+	case STATE_RESPONSE_CHUNKED_CRLF:
 	case STATE_RESPONSE_CHUNKED_TRAILER:
 		return http_read(conn);
 	case STATE_WRITE_DATA:
@@ -1817,8 +1839,12 @@ proc_http(char *bind_addr, int fd)
 			if (i >= NPFDS)
 				errx(1, "too many connections");
 
-			if (conn->io_time == 0)
-				conn->io_time = now + HTTP_IO_TIMEOUT;
+			if (conn->io_time == 0) {
+				if (conn->state == STATE_CONNECT)
+					conn->io_time = now + MAX_CONN_TIMEOUT;
+				else
+					conn->io_time = now + MAX_IO_TIMEOUT;
+			}
 
 			if (conn->io_time <= now)
 				timeout = 0;
@@ -1906,9 +1932,15 @@ proc_http(char *bind_addr, int fd)
 			if (conn->pfd != NULL && conn->pfd->revents != 0)
 				http_do(conn, http_handle);
 			else if (conn->io_time <= now) {
-				warnx("%s: timeout, connection closed",
-				    http_info(conn->host));
-				http_do(conn, http_failed);
+				if (conn->state == STATE_CONNECT) {
+					warnx("%s: connect timeout",
+					    http_info(conn->host));
+					http_do(conn, http_connect_failed);
+				} else {
+					warnx("%s: timeout, connection closed",
+					    http_info(conn->host));
+					http_do(conn, http_failed);
+				}
 			}
 
 			if (conn->state == STATE_FREE)
