@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.208 2022/06/27 10:18:27 job Exp $ */
+/*	$OpenBSD: main.c,v 1.219 2022/09/03 09:22:25 claudio Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -66,6 +66,7 @@ int	noop;
 int	filemode;
 int	rrdpon = 1;
 int	repo_timeout;
+time_t	deadline;
 
 struct skiplist skiplist = LIST_HEAD_INITIALIZER(skiplist);
 
@@ -256,6 +257,18 @@ rrdp_fetch(unsigned int id, const char *uri, const char *local,
 	io_close_buffer(&rrdpq, b);
 }
 
+void
+rrdp_abort(unsigned int id)
+{
+	enum rrdp_msg type = RRDP_ABORT;
+	struct ibuf *b;
+
+	b = io_new_buffer();
+	io_simple_buffer(b, &type, sizeof(type));
+	io_simple_buffer(b, &id, sizeof(id));
+	io_close_buffer(&rrdpq, b);
+}
+
 /*
  * Request a repository sync via rsync URI to directory local.
  */
@@ -273,6 +286,19 @@ rsync_fetch(unsigned int id, const char *uri, const char *local,
 	io_close_buffer(&rsyncq, b);
 }
 
+void
+rsync_abort(unsigned int id)
+{
+	struct ibuf	*b;
+
+	b = io_new_buffer();
+	io_simple_buffer(b, &id, sizeof(id));
+	io_str_buffer(b, NULL);
+	io_str_buffer(b, NULL);
+	io_str_buffer(b, NULL);
+	io_close_buffer(&rsyncq, b);
+}
+
 /*
  * Request a file from a https uri, data is written to the file descriptor fd.
  */
@@ -333,12 +359,14 @@ rrdp_http_done(unsigned int id, enum http_result res, const char *last_mod)
  * These are always relative to the directory in which "mft" sits.
  */
 static void
-queue_add_from_mft(const struct mft *mft, struct repo *rp)
+queue_add_from_mft(const struct mft *mft)
 {
 	size_t			 i;
+	struct repo		*rp;
 	const struct mftfile	*f;
 	char			*nfile, *npath = NULL;
 
+	rp = repo_byid(mft->repoid);
 	for (i = 0; i < mft->filesz; i++) {
 		f = &mft->files[i];
 
@@ -475,13 +503,14 @@ queue_add_from_cert(const struct cert *cert)
  */
 static void
 entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
-    struct brk_tree *brktree)
+    struct brk_tree *brktree, struct vap_tree *vaptree)
 {
 	enum rtype	 type;
 	struct tal	*tal;
 	struct cert	*cert;
 	struct mft	*mft;
 	struct roa	*roa;
+	struct aspa	*aspa;
 	char		*file;
 	int		 c;
 
@@ -541,7 +570,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 		}
 		mft = mft_read(b);
 		if (!mft->stale)
-			queue_add_from_mft(mft, repo_byid(mft->repoid));
+			queue_add_from_mft(mft);
 		else
 			st->mfts_stale++;
 		mft_free(mft);
@@ -568,6 +597,21 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 		break;
 	case RTYPE_FILE:
 		break;
+	case RTYPE_ASPA:
+		st->aspas++;
+		io_read_buf(b, &c, sizeof(c));
+		if (c == 0) {
+			st->aspas_fail++;
+			break;
+		}
+		aspa = aspa_read(b);
+		if (aspa->valid)
+			aspa_insert_vaps(vaptree, aspa, &st->vaps,
+			    &st->vaps_uniqs);
+		else
+			st->aspas_invalid++;
+		aspa_free(aspa);
+		break;
 	default:
 		errx(1, "unknown entity type %d", type);
 	}
@@ -674,16 +718,15 @@ load_skiplist(const char *slf)
 	struct skiplistentry	*sle;
 	FILE			*fp;
 	char			*line = NULL;
-	size_t			 linesize = 0, s;
-	ssize_t			 linelen;
+	size_t			 linesize = 0, linelen;
 
 	if ((fp = fopen(slf, "r")) == NULL) {
-		if (strcmp(slf, DEFAULT_SKIPLIST_FILE) != 0)
-			errx(1, "failed to open skiplist %s", slf);
-		return;
+		if (errno == ENOENT && strcmp(slf, DEFAULT_SKIPLIST_FILE) == 0)
+			return;
+		err(1, "failed to open %s", slf);
 	}
 
-	while ((linelen = getline(&line, &linesize, fp)) != -1) {
+	while (getline(&line, &linesize, fp) != -1) {
 		/* just eat comment lines or empty lines*/
 		if (line[0] == '#' || line[0] == '\n')
 			continue;
@@ -695,12 +738,11 @@ load_skiplist(const char *slf)
 		 * Ignore anything after comment sign, whitespaces,
 		 * also chop off LF or CR.
 		 */
-		line[strcspn(line, " #\r\n\t")] = 0;
+		linelen = strcspn(line, " #\r\n\t");
+		line[linelen] = '\0';
 
-		for (s = 0; s < strlen(line); s++)
-			if (!isalnum((unsigned char)line[s]) &&
-			    !ispunct((unsigned char)line[s]))
-				errx(1, "invalid entry in skiplist: %s", line);
+		if (!valid_uri(line, linelen, NULL))
+			errx(1, "invalid entry in skiplist: %s", line);
 
 		if ((sle = malloc(sizeof(struct skiplistentry))) == NULL)
 			err(1, NULL);
@@ -793,6 +835,7 @@ main(int argc, char *argv[])
 	const char	*skiplistfile = NULL;
 	struct vrp_tree	 vrps = RB_INITIALIZER(&vrps);
 	struct brk_tree	 brks = RB_INITIALIZER(&brks);
+	struct vap_tree	 vaps = RB_INITIALIZER(&vaps);
 	struct rusage	 ru;
 	struct timeval	 start_time, now_time;
 
@@ -1006,10 +1049,13 @@ main(int argc, char *argv[])
 		 */
 		alarm(timeout);
 		signal(SIGALRM, suicide);
+
+		/* give up a bit before the hard timeout and try to finish up */
+		if (!noop)
+			deadline = getmonotime() + timeout - repo_timeout / 2;
 	}
 
-	/* TODO unveil cachedir and outputdir, no other access allowed */
-	if (pledge("stdio rpath wpath cpath fattr sendfd", NULL) == -1)
+	if (pledge("stdio rpath wpath cpath fattr sendfd unveil", NULL) == -1)
 		err(1, "pledge");
 
 	msgbuf_init(&procq);
@@ -1050,7 +1096,17 @@ main(int argc, char *argv[])
 	if (filemode) {
 		while (*argv != NULL)
 			queue_add_file(*argv++, RTYPE_FILE, 0);
+
+		if (unveil(cachedir, "r") == -1)
+			err(1, "unveil cachedir");
+	} else {
+		if (unveil(outputdir, "rwc") == -1)
+			err(1, "unveil outputdir");
+		if (unveil(cachedir, "rwc") == -1)
+			err(1, "unveil cachedir");
 	}
+	if (pledge("stdio rpath wpath cpath fattr sendfd", NULL) == -1)
+		err(1, "unveil");
 
 	/* change working directory to the cache directory */
 	if (fchdir(cachefd) == -1)
@@ -1067,7 +1123,7 @@ main(int argc, char *argv[])
 
 		polltim = repo_check_timeout(INFTIM);
 
-		if ((c = poll(pfd, NPFD, polltim)) == -1) {
+		if (poll(pfd, NPFD, polltim) == -1) {
 			if (errno == EINTR)
 				continue;
 			err(1, "poll");
@@ -1152,7 +1208,7 @@ main(int argc, char *argv[])
 		if ((pfd[0].revents & POLLIN)) {
 			b = io_buf_read(proc, &procbuf);
 			if (b != NULL) {
-				entity_process(b, &stats, &vrps, &brks);
+				entity_process(b, &stats, &vrps, &brks, &vaps);
 				ibuf_free(b);
 			}
 		}
@@ -1235,7 +1291,7 @@ main(int argc, char *argv[])
 	if (fchdir(outdirfd) == -1)
 		err(1, "fchdir output dir");
 
-	if (outputfiles(&vrps, &brks, &stats))
+	if (outputfiles(&vrps, &brks, &vaps, &stats))
 		rc = 1;
 
 	printf("Processing time %lld seconds "
@@ -1244,8 +1300,10 @@ main(int argc, char *argv[])
 	    (long long)stats.user_time.tv_sec,
 	    (long long)stats.system_time.tv_sec);
 	printf("Skiplist entries: %zu\n", stats.skiplistentries);
-	printf("Route Origin Authorizations: %zu (%zu failed parse, %zu invalid)\n",
-	    stats.roas, stats.roas_fail, stats.roas_invalid);
+	printf("Route Origin Authorizations: %zu (%zu failed parse, %zu "
+	    "invalid)\n", stats.roas, stats.roas_fail, stats.roas_invalid);
+	printf("AS Provider Attestations: %zu (%zu failed parse, %zu "
+	    "invalid)\n", stats.aspas, stats.aspas_fail, stats.aspas_invalid);
 	printf("BGPsec Router Certificates: %zu\n", stats.brks);
 	printf("Certificates: %zu (%zu invalid)\n",
 	    stats.certs, stats.certs_fail);
@@ -1259,6 +1317,7 @@ main(int argc, char *argv[])
 	printf("Cleanup: removed %zu files, %zu directories, %zu superfluous\n",
 	    stats.del_files, stats.del_dirs, stats.extra_files);
 	printf("VRP Entries: %zu (%zu unique)\n", stats.vrps, stats.uniqs);
+	printf("VAP Entries: %zu (%zu unique)\n", stats.vaps, stats.vaps_uniqs);
 
 	/* Memory cleanup. */
 	repo_free();
@@ -1271,6 +1330,7 @@ usage:
 	    " [-e rsync_prog]\n"
 	    "                   [-S skiplist] [-s timeout] [-T table] [-t tal]"
 	    " [outputdir]\n"
-	    "       rpki-client [-Vv] [-d cachedir] [-t tal] -f file ...\n");
+	    "       rpki-client [-Vv] [-d cachedir] [-j] [-t tal] -f file ..."
+	    "\n");
 	return 1;
 }
