@@ -1,4 +1,4 @@
-/*	$OpenBSD: print.c,v 1.13 2022/05/15 16:43:34 tb Exp $ */
+/*	$OpenBSD: print.c,v 1.16 2022/08/30 23:41:53 tb Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -91,15 +91,15 @@ tal_print(const struct tal *p)
 
 	if (outformats & FORMAT_JSON) {
 		printf("\t\"type\": \"tal\",\n");
-		printf("\t\"name\": %s\n", p->descr);
-		printf("\t\"ski\": %s\n", pretty_key_id(ski));
+		printf("\t\"name\": \"%s\",\n", p->descr);
+		printf("\t\"ski\": \"%s\",\n", pretty_key_id(ski));
 		printf("\t\"trust_anchor_locations\": [");
 		for (i = 0; i < p->urisz; i++) {
 			printf("\"%s\"", p->uri[i]);
 			if (i + 1 < p->urisz)
 				printf(", ");
 		}
-		printf("]\n");
+		printf("],\n");
 	} else {
 		printf("Trust anchor name: %s\n", p->descr);
 		printf("Subject key identifier: %s\n", pretty_key_id(ski));
@@ -568,3 +568,52 @@ rsc_print(const X509 *x, const struct rsc *p)
 	if (outformats & FORMAT_JSON)
 		printf("\t],\n");
 }
+
+void
+aspa_print(const X509 *x, const struct aspa *p)
+{
+	size_t	i;
+
+	if (outformats & FORMAT_JSON) {
+		printf("\t\"type\": \"aspa\",\n");
+		printf("\t\"ski\": \"%s\",\n", pretty_key_id(p->ski));
+		x509_print(x);
+		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
+		printf("\t\"aia\": \"%s\",\n", p->aia);
+		printf("\t\"customer_asid\": %u,\n", p->custasid);
+		printf("\t\"provider_set\": [\n");
+		for (i = 0; i < p->providersz; i++) {
+			printf("\t\t{ \"asid\": %u", p->providers[i].as);
+			if (p->providers[i].afi == AFI_IPV4)
+				printf(", \"afi_limit\": \"ipv4\"");
+			if (p->providers[i].afi == AFI_IPV6)
+				printf(", \"afi_limit\": \"ipv6\"");
+			printf(" }");
+			if (i + 1 < p->providersz)
+				printf(",");
+			printf("\n");
+		}
+		printf("\t],\n");
+	} else {
+		printf("Subject key identifier: %s\n", pretty_key_id(p->ski));
+		x509_print(x);
+		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
+		printf("Authority info access: %s\n", p->aia);
+		printf("Customer AS: %u\n", p->custasid);
+		printf("Provider Set:\n");
+		for (i = 0; i < p->providersz; i++) {
+			printf("%5zu: AS: %d", i + 1, p->providers[i].as);
+			switch (p->providers[i].afi) {
+			case AFI_IPV4:
+				printf(" (IPv4 only)");
+				break;
+			case AFI_IPV6:
+				printf(" (IPv6 only)");
+				break;
+			default:
+				break;
+			}
+			printf("\n");
+		}
+	}
+}
