@@ -1,4 +1,4 @@
-/*	$OpenBSD: rrdp_notification.c,v 1.14 2022/02/14 14:46:16 job Exp $ */
+/*	$OpenBSD: rrdp_notification.c,v 1.16 2022/06/16 16:09:56 claudio Exp $ */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
@@ -383,8 +383,13 @@ notification_done(struct notification_xml *nxml, char *last_mod)
 	if (nxml->repository->serial == 0)
 		goto snapshot;
 
-	/* if our serial is equal or bigger, the repo is up to date */
-	if (nxml->repository->serial >= nxml->serial) {
+	if (nxml->repository->serial > nxml->serial)
+		warnx("%s: serial number decreased from %lld to %lld",
+		    nxml->notifyuri, nxml->repository->serial, nxml->serial);
+
+	/* if our serial is equal or plus 2, the repo is up to date */
+	if (nxml->repository->serial >= nxml->serial &&
+	    nxml->repository->serial - nxml->serial <= 2) {
 		nxml->current->serial = nxml->repository->serial;
 		return NOTIFICATION;
 	}
@@ -431,7 +436,7 @@ notification_get_next(struct notification_xml *nxml, char *hash, size_t hlen,
 		nxml->current->serial = nxml->serial;
 		return nxml->snapshot_uri;
 	case DELTA:
-		/* first bump serial, then use first delta  */
+		/* first bump serial, then use first delta */
 		nxml->current->serial += 1;
 		d = TAILQ_FIRST(&nxml->delta_q);
 		assert(d->serial == nxml->current->serial);
