@@ -1,4 +1,4 @@
-/*	$OpenBSD: x509.c,v 1.46 2022/05/31 18:51:35 tb Exp $ */
+/*	$OpenBSD: x509.c,v 1.50 2022/09/03 14:40:09 job Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
@@ -44,6 +44,7 @@ ASN1_OBJECT	*msg_dgst_oid;	/* pkcs-9 id-messageDigest */
 ASN1_OBJECT	*sign_time_oid;	/* pkcs-9 id-signingTime */
 ASN1_OBJECT	*bin_sign_time_oid;	/* pkcs-9 id-aa-binarySigningTime */
 ASN1_OBJECT	*rsc_oid;	/* id-ct-signedChecklist */
+ASN1_OBJECT	*aspa_oid;	/* id-ct-ASPA */
 
 void
 x509_init_oid(void)
@@ -81,6 +82,9 @@ x509_init_oid(void)
 	if ((rsc_oid = OBJ_txt2obj("1.2.840.113549.1.9.16.1.48", 1)) == NULL)
 		errx(1, "OBJ_txt2obj for %s failed",
 		    "1.2.840.113549.1.9.16.1.48");
+	if ((aspa_oid = OBJ_txt2obj("1.2.840.113549.1.9.16.1.49", 1)) == NULL)
+		errx(1, "OBJ_txt2obj for %s failed",
+		    "1.2.840.113549.1.9.16.1.49");
 }
 
 /*
@@ -348,7 +352,7 @@ x509_get_expire(X509 *x, const char *fn, time_t *tt)
 }
 
 /*
- * Check whether the RFC 3779 extensions are set to inherit.
+ * Check whether all RFC 3779 extensions are set to inherit.
  * Return 1 if both AS & IP are set to inherit.
  * Return 0 on failure (such as missing extensions or no inheritance).
  */
@@ -392,6 +396,32 @@ x509_inherits(X509 *x)
 	return rc;
 }
 
+/*
+ * Check whether at least one RFC 3779 extension is set to inherit.
+ * Return 1 if an inherit element is encountered in AS or IP.
+ * Return 0 otherwise.
+ */
+int
+x509_any_inherits(X509 *x)
+{
+	STACK_OF(IPAddressFamily)	*addrblk = NULL;
+	ASIdentifiers			*asidentifiers = NULL;
+	int				 rc = 0;
+
+	addrblk = X509_get_ext_d2i(x, NID_sbgp_ipAddrBlock, NULL, NULL);
+	if (X509v3_addr_inherits(addrblk))
+		rc = 1;
+
+	asidentifiers = X509_get_ext_d2i(x, NID_sbgp_autonomousSysNum, NULL,
+	    NULL);
+	if (X509v3_asid_inherits(asidentifiers))
+		rc = 1;
+
+	ASIdentifiers_free(asidentifiers);
+	sk_IPAddressFamily_pop_free(addrblk, IPAddressFamily_free);
+	return rc;
+}
+
 /*
  * Parse the very specific subset of information in the CRL distribution
  * point extension.
@@ -536,7 +566,7 @@ x509_get_time(const ASN1_TIME *at, time_t *t)
 }
 
 /*
- * Extract and validate an accessLocation, RFC 6487, 4.8 and RFC 8192, 3.2.
+ * Extract and validate an accessLocation, RFC 6487, 4.8 and RFC 8182, 3.2.
  * Returns 0 on failure and 1 on success.
  */
 int
@@ -545,12 +575,6 @@ x509_location(const char *fn, const char *descr, const char *proto,
 {
 	ASN1_IA5STRING	*uri;
 
-	if (*out != NULL) {
-		warnx("%s: RFC 6487 section 4.8: %s already specified", fn,
-		    descr);
-		return 0;
-	}
-
 	if (location->type != GEN_URI) {
 		warnx("%s: RFC 6487 section 4.8: %s not URI", fn, descr);
 		return 0;
@@ -563,6 +587,12 @@ x509_location(const char *fn, const char *descr, const char *proto,
 		return 0;
 	}
 
+	if (*out != NULL) {
+		warnx("%s: RFC 6487 section 4.8: multiple %s specified, "
+		    "using the first one", fn, descr);
+		return 1;
+	}
+
 	if ((*out = strndup(uri->data, uri->length)) == NULL)
 		err(1, NULL);
 
